#[derive(Debug, Clone)]
#[cfg_attr(feature = "ssr", derive(serde::Serialize, serde::Deserialize))]
pub struct PostMetadata {
    image_path: Option<String>,
    title: String,
    date: String,
    description: String,
    project_link: Option<String>,
}

#[derive(Debug, Clone)]
#[cfg_attr(feature = "ssr", derive(serde::Serialize, serde::Deserialize))]
pub struct Post {
    metadata: PostMetadata,
    content: String,
}

cfg_if::cfg_if! {
    if #[cfg(feature = "ssr")] {
        impl From<String> for Post {
            fn from(content: String) -> Self {
                use gray_matter::{Matter, engine::YAML};
                let matter = Matter::<YAML>::new();

                let post_data = matter
                    .parse_with_struct::<PostMetadata>(&content)
                    .expect("Unable to parse md frontmatter");

                let metadata = post_data.data;
                let content = post_data.content;

                use pulldown_cmark::{Parser, Options, html};
                let parser = Parser::new_ext(&content, Options::all());
                let mut html_output = String::new();
                html::push_html(&mut html_output, parser);

                Self {
                    metadata,
                    content: html_output,
                }
            }
        }

        impl Post {
            pub fn get_all(folder: &str) -> Result<Vec<Self>, String> {
                use std::{path::Path, fs::read_dir};

                let mut posts: Vec<Self> = Vec::new();

                let folder_path = Path::new("markdowns").join(folder);
                let paths = read_dir(folder_path).map_err(|e| e.to_string())?;

                for path_result in paths {
                    let path = match path_result {
                        Ok(path) => path.path(),
                        Err(e) => {
                            eprintln!("{:?}", e);
                            continue;
                        }
                    };

                    let content = std::fs::read_to_string(path);
                    let content = match content {
                        Ok(content) => content,
                        Err(e) => {
                            eprintln!("{:?}", e);
                            continue;
                        }
                    };
                    let post = Self::from(content);
                    posts.push(post);
                }

                Ok(posts)
            }
        }
    }
}

